/*
 * $Id: rr.c,v 1.4 2003/04/26 03:24:16 kenta Exp $
 *
 * Copyright 2003 Kenta Cho. All rights reserved.
 */

/**
 * rRootage main routine.
 *
 * @version $Revision: 1.4 $
 */
#include "SDL.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "rr.h"
#include "screen.h"
#include "vector.h"
#include "foe_mtd.h"
#include "brgmng_mtd.h"
#include "degutil.h"
#include "boss_mtd.h"
#include "ship.h"
#include "laser.h"
#include "frag.h"
#include "shot.h"
#include "background.h"
#include "soundmanager.h"
#include "attractmanager.h"

static int noSound = 0;

// Initialize and load preference.
static void initFirst() {
  time_t timer;
  time(&timer);
  srand(timer);

  loadPreference();
  initBarragemanager();
  initAttractManager();
  if ( !noSound ) initSound();
  initGameStateFirst();
}

// Quit and save preference.
void quitLast() {
  if ( !noSound ) closeSound();
  savePreference();
  closeFoes();
  closeBarragemanager();
  closeSDL();
  SDL_Quit();
  exit(1);
}

int status;

void initTitleStage(int stg) {
  initFoes();
  initStageState(stg);
}

void initTitle() {
  int stg;
  status = TITLE;

  stg = initTitleAtr();
  initBoss();
  initShip();
  initLasers();
  initFrags();
  initShots();
  initBackground(0);
  initTitleStage(stg);
  left = -1;
}

void initGame(int stg) {
  int sn;
  status = IN_GAME;

  initBoss();
  initFoes();
  initShip();
  initLasers();
  initFrags();
  initShots();

  initGameState(stg);
  sn = stg%SAME_RANK_STAGE_NUM;
  initBackground(sn);
  if ( sn == SAME_RANK_STAGE_NUM-1 ) {
    playMusic(rand()%(SAME_RANK_STAGE_NUM-1));
  } else {
    playMusic(sn);
  }
}

void initGameover() {
  status = GAMEOVER;
  initGameoverAtr();
}

static void move() {
  switch ( status ) {
  case TITLE:
    moveTitleMenu();
    moveBoss();
    moveFoes();
    moveBackground();
    break;
  case IN_GAME:
  case STAGE_CLEAR:
    moveShip();
    moveBoss();
    moveLasers();
    moveShots();
    moveFoes();
    moveFrags();
    moveBackground();
    break;
  case GAMEOVER:
    moveGameover();
    moveBoss();
    moveFoes();
    moveFrags();
    moveBackground();
    break;
  case PAUSE:
    movePause();
    break;
  }
  moveScreenShake();
}


//NOTE - senquack - when I disable everything but drawBackground here it will not freeze:
// ANOTHER NOTE: simply disabling the calls to drawBoss here allow it to run much longer:
//senquack - tried tweaking this to fix hang:
//static void draw() {
//  switch ( status ) {
//  case TITLE:
//  	//printf("draw(): Drawing TITLE\n");
//  	//printf("draw(): Drawing TITLE - drawBackground\n");
//    drawBackground();
//    //printf("draw(): Drawing TITLE - drawBoss\n");
//    drawBoss();
//    //printf("draw(): Drawing TITLE - drawBulletsWake\n");
//    drawBulletsWake();
//    
//    //TODO: FIX CRASH IN drawBulletsWake() !!!!!!!!!!!!!!!!! **********
//    // When drawBulletsWake used (or buttons pushed), it crashes?
//    
//    
//    //printf("draw(): Drawing TITLE - drawBullets\n");
//    drawBullets();
//    //printf("draw(): Drawing TITLE - startDrawBoards\n");
//    startDrawBoards();
//    //printf("draw(): Drawing TITLE - drawSideBoards\n");
//    drawSideBoards();
//    //printf("draw(): Drawing TITLE - drawTitle\n");
//    drawTitle();
//    //printf("draw(): Drawing TITLE - endDrawBoards\n");
//    endDrawBoards();
//    //printf("draw(): Drawing TITLE - drawTestPoly\n");
//    //drawTestPoly();
//    break;
//  case IN_GAME:
//  case STAGE_CLEAR:
//  	printf("draw(): Drawing STAGE_CLEAR\n");
//    drawBackground();
//    drawBoss();
//    drawLasers();
//    drawShots();
//    drawBulletsWake();
//    drawFrags();
//    drawShip();
//    drawBullets();
//    startDrawBoards();
//    drawSideBoards();
//    drawBossState();
//    endDrawBoards();
//    break;
//  case GAMEOVER:
//  	printf("draw(): Drawing GAMEOVER\n");
//    drawBackground();
//    drawBoss();
//    drawBulletsWake();
//    drawFrags();
//    drawBullets();
//    startDrawBoards();
//    drawSideBoards();
//    drawGameover();
//    endDrawBoards();
//    break;
//  case PAUSE:
//  	printf("draw(): Drawing PAUSE\n");
//    drawBackground();
//    drawBoss();
//    drawLasers();
//    drawShots();
//    drawBulletsWake();
//    drawFrags();
//    drawShip();
//    drawBullets();
//    startDrawBoards();
//    drawSideBoards();
//    drawBossState();
//    drawPause();
//    endDrawBoards();
//    break;
//  }
//}
static void draw() {
  switch ( status ) {
  case TITLE:
  	//printf("draw(): Drawing TITLE\n");
  	//printf("draw(): Drawing TITLE - drawBackground\n");
//    drawBackground();
    //printf("draw(): Drawing TITLE - drawBoss\n");
    drawBoss();
    //printf("draw(): Drawing TITLE - drawBulletsWake\n");
//    drawBulletsWake();
    
    //TODO: FIX CRASH IN drawBulletsWake() !!!!!!!!!!!!!!!!! **********
    // When drawBulletsWake used (or buttons pushed), it crashes?
    
    
    //printf("draw(): Drawing TITLE - drawBullets\n");
    drawBullets();
    //printf("draw(): Drawing TITLE - startDrawBoards\n");
    startDrawBoards();
    //printf("draw(): Drawing TITLE - drawSideBoards\n");
    drawSideBoards();
    //printf("draw(): Drawing TITLE - drawTitle\n");
    drawTitle();
    //printf("draw(): Drawing TITLE - endDrawBoards\n");
    endDrawBoards();
    //printf("draw(): Drawing TITLE - drawTestPoly\n");
//    drawTestPoly();
    break;
  case IN_GAME:
  case STAGE_CLEAR:
	 //senquack
//  	printf("draw(): Drawing STAGE_CLEAR\n");
//    drawBackground();
	 // NOTE - senquack: just drawShip and drawBullets here fixes freezes, enabling drawBoss
	 //		causes freezing very quickly
    drawBoss();
    drawLasers();
    drawShots();
//    drawBulletsWake();
//    drawFrags();
    drawShip();
    drawBullets();
    startDrawBoards();
    drawSideBoards();
    drawBossState();
    endDrawBoards();
    break;
  case GAMEOVER:
	 //senquack
//  	printf("draw(): Drawing GAMEOVER\n");
//    drawBackground();
    drawBoss();
//    drawBulletsWake();
//    drawFrags();
    drawBullets();
    startDrawBoards();
    drawSideBoards();
    drawGameover();
    endDrawBoards();
    break;
  case PAUSE:
	 //senquack
//  	printf("draw(): Drawing PAUSE\n");
//    drawBackground();
    drawBoss();
    drawLasers();
    drawShots();
//    drawBulletsWake();
//    drawFrags();
    drawShip();
    drawBullets();
    startDrawBoards();
    drawSideBoards();
    drawBossState();
    drawPause();
    endDrawBoards();
    break;
  }
}

static int accframe = 0;

static void usage(char *argv0) {
  fprintf(stderr, "Usage: %s [-laser] [-nosound] [-reverse] [-nowait] [-accframe]\n", argv0);
}

static void parseArgs(int argc, char *argv[]) {
  int i;
  for ( i=1 ; i<argc ; i++ ) {
//    if ( strcmp(argv[i], "-lowres") == 0 ) {
//      lowres = 1;
//    } else if ( strcmp(argv[i], "-nosound") == 0 ) {
    if ( strcmp(argv[i], "-nosound") == 0 ) {
      noSound = 1;
//    } else if ( strcmp(argv[i], "-window") == 0 ) {
//      windowMode = 1;
    } else if ( strcmp(argv[i], "-reverse") == 0 ) {
      buttonReversed = 1;
    }
    /* else if ( (strcmp(argv[i], "-brightness") == 0) && argv[i+1] ) {
      i++;
      brightness = (int)atoi(argv[i]);
      if ( brightness < 0 || brightness > 256 ) {
	brightness = DEFAULT_BRIGHTNESS;
      }
      }*/ 
    else if ( strcmp(argv[i], "-nowait") == 0 ) {
      nowait = 1;
    } else if ( strcmp(argv[i], "-accframe") == 0 ) {
      accframe = 1;
    } else if ( strcmp(argv[i], "-laser") == 0 ) {
      laserOnByDefault = 1;
    } else {
      usage(argv[0]);
      exit(1);
    }
  }
}

int interval = INTERVAL_BASE;
int tick = 0;
static int pPrsd = 1;

int main(int argc, char *argv[]) {

  int done = 0;

  long prvTickCount = 0;
  int i;
  SDL_Event event;
  long nowTick;
  int frame;


  //senquack - for FPS computation
  int fpsctr_oldticks = 0;
  int fpsctr_newticks = 0;
  int fpsctr_frames = 0;

  parseArgs(argc, argv);

  initDegutil();
  initSDL();
  initFirst();
  initTitle();

  //senquack
//printf("Init title done\n");
//fflush(stdout);

  while ( !done ) {
    SDL_PollEvent(&event);
//    keys = SDL_GetKeyState(NULL);


	 //senquack - all this button handling sure is a ugly mess, let's just hack it and try to forget
//    if ( keys[SDLK_ESCAPE] == SDL_PRESSED || event.type == SDL_QUIT ) done = 1;
//    if ( keys[SDLK_p] == SDL_PRESSED ) {
//      if ( !pPrsd ) {
//			if ( status == IN_GAME ) {
//			  status = PAUSE;
//			} else if ( status == PAUSE ) {
//			  status = IN_GAME;
//			}
//      }
//      pPrsd = 1;
//    } else {
//      pPrsd = 0;
//    }

	 //senquack - Quitting is handled from the main menu.  The quit button merely exits to the
	 //				main menu and that logic is handled elsewhere now.
    if ( SDL_JoystickGetButton(stick, GP2X_BUTTON_START)) {
      if ( !pPrsd ) {
			if ( status == IN_GAME ) {
			  status = PAUSE;
			} else if ( status == PAUSE ) {
			  status = IN_GAME;
			}
      }
      pPrsd = 1;
    } else {
      pPrsd = 0;
    }

	 //senquack
	 if ( SDL_JoystickGetButton(stick, GP2X_BUTTON_SELECT) && status == IN_GAME) {
		 initGameover();
	 }

    nowTick = SDL_GetTicks();
    frame = (int)(nowTick-prvTickCount) / interval;
    if ( frame <= 0 ) {
      frame = 1;
      SDL_Delay(prvTickCount+interval-nowTick);
      if ( accframe ) {
	prvTickCount = SDL_GetTicks();
      } else {
	prvTickCount += interval;
      }
    } else if ( frame > 5 ) {
      frame = 5;
      prvTickCount = nowTick;
    } else {
      prvTickCount += frame*interval;
    }
    for ( i=0 ; i<frame ; i++ ) {
      move();
      tick++;
    }

	 //senquack - display FPS so we can optimize
	 fpsctr_frames++;
	 fpsctr_newticks = SDL_GetTicks();
	 if ((fpsctr_newticks - fpsctr_oldticks) >= 1000)
	 {
		 printf("\nFPS: %d\n", fpsctr_frames);	fflush(stdout);
		 fpsctr_frames = 0;
		 fpsctr_oldticks = fpsctr_newticks;
	 }


//printf("Starting to draw\n");
//fflush(stdout);

    drawGLSceneStart();
//printf("drawGLSceneStart done\n");
    draw();
//printf("draw done\n");    
    drawGLSceneEnd();
//printf("drawGLSceneEnd done\n");    
    swapGLScene();
//printf("swapGLScene done\n");    

	 //senquack - experiment
//	 SDL_Delay(40);
  }
  printf("variable \"done\": %d\n", done);
  quitLast();
  return 0;
}
